"use strict";
/**
 * Copyright 2023 actions-toolkit authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Bake = void 0;
const buildx_1 = require("./buildx");
const exec_1 = require("../exec");
const inputs_1 = require("./inputs");
const util_1 = require("../util");
class Bake {
    buildx;
    constructor(opts) {
        this.buildx = opts?.buildx || new buildx_1.Buildx();
    }
    async getDefinition(cmdOpts, execOptions) {
        execOptions = execOptions || { ignoreReturnCode: true };
        execOptions.ignoreReturnCode = true;
        if (cmdOpts.githubToken) {
            execOptions.env = Object.assign({}, process.env, {
                BUILDX_BAKE_GIT_AUTH_TOKEN: cmdOpts.githubToken
            });
        }
        const args = ['bake'];
        let remoteDef;
        const files = [];
        const sources = [...(cmdOpts.files || []), cmdOpts.source];
        if (sources) {
            for (const source of sources.map(v => (v ? v.trim() : ''))) {
                if (source.length == 0) {
                    continue;
                }
                if (!util_1.Util.isValidRef(source)) {
                    files.push(source);
                    continue;
                }
                if (remoteDef) {
                    throw new Error(`Only one remote bake definition can be defined`);
                }
                remoteDef = source;
            }
        }
        if (remoteDef) {
            args.push(remoteDef);
        }
        for (const file of files) {
            args.push('--file', file);
        }
        if (cmdOpts.overrides) {
            for (const override of cmdOpts.overrides) {
                args.push('--set', override);
            }
        }
        if (cmdOpts.load) {
            args.push('--load');
        }
        if (cmdOpts.noCache) {
            args.push('--no-cache');
        }
        if (cmdOpts.provenance) {
            args.push('--provenance', cmdOpts.provenance);
        }
        if (cmdOpts.push) {
            args.push('--push');
        }
        if (cmdOpts.sbom) {
            args.push('--sbom', cmdOpts.sbom);
        }
        const printCmd = await this.buildx.getCommand([...args, '--print', ...(cmdOpts.targets || [])]);
        return await exec_1.Exec.getExecOutput(printCmd.command, printCmd.args, execOptions).then(res => {
            if (res.stderr.length > 0 && res.exitCode != 0) {
                throw new Error(`cannot parse bake definitions: ${res.stderr.match(/(.*)\s*$/)?.[0]?.trim() ?? 'unknown error'}`);
            }
            return Bake.parseDefinition(res.stdout.trim());
        });
    }
    static parseDefinition(dt) {
        return JSON.parse(dt);
    }
    static hasLocalExporter(def) {
        return inputs_1.Inputs.hasExporterType('local', Bake.exporters(def));
    }
    static hasTarExporter(def) {
        return inputs_1.Inputs.hasExporterType('tar', Bake.exporters(def));
    }
    static hasDockerExporter(def, load) {
        return load || inputs_1.Inputs.hasExporterType('docker', Bake.exporters(def));
    }
    static exporters(def) {
        const exporters = new Array();
        for (const key in def.target) {
            const target = def.target[key];
            if (target.output) {
                exporters.push(...target.output);
            }
        }
        return exporters;
    }
}
exports.Bake = Bake;
//# sourceMappingURL=bake.js.map